package main

import (
	"flag"
	"io/ioutil"
	"log"
	"path"
	"strings"
	"sync"

	"github.com/upyun/go-sdk/upyun"
)

var (
	bucket   string
	username string
	password string
	dir      string
	filename string
	workers  int

	fileChan chan string
	dirChan  chan string

	lock sync.Mutex

	up *upyun.UpYun
)

func doListLeaf(p string) bool {
	log.Println("doListLeaf", p)
	objChan := make(chan *upyun.FileInfo, 1000000)
	go func() {
		err := up.List(&upyun.GetObjectsConfig{
			Path:         p,
			ObjectsChan:  objChan,
			MaxListLevel: -1,
		})
		if err != nil {
			log.Println(p, err)
			return
		}
	}()

	has := false
	for obj := range objChan {
		if !obj.IsDir || obj.IsEmptyDir {
			rp := path.Join(p, obj.Name)
			fileChan <- rp
			has = true
		}
	}
	return has
}

func consume() {
	for file := range fileChan {
		err := up.Delete(&upyun.DeleteObjectConfig{
			Path: file,
		})
		log.Println(file, err)
	}
}

func doListAll(ps []string) []string {
	rest := []string{}
	wg := sync.WaitGroup{}
	for _, p := range ps {
		wg.Add(1)
		go func(pp string) {
			has := doListLeaf(pp)
			if has {
				lock.Lock()
				rest = append(rest, pp)
				lock.Unlock()
			}
			wg.Done()
		}(p)
	}
	wg.Wait()
	close(fileChan)
	return rest
}

func doDeleteV2(ps []string) []string {
	fileChan = make(chan string, 10000)
	wg := sync.WaitGroup{}
	for i := 0; i < workers; i++ {
		wg.Add(1)
		go func() {
			consume()
			wg.Done()
		}()
	}

	rest := doListAll(ps)
	wg.Wait()

	return rest
}

func main() {
	flag.StringVar(&bucket, "b", "", "bucket name")
	flag.StringVar(&username, "u", "", "username")
	flag.StringVar(&password, "p", "", "password")
	flag.StringVar(&dir, "d", "", "directory names")
	flag.IntVar(&workers, "w", 10, "workers")
	flag.StringVar(&filename, "f", "", "filename")
	flag.Parse()

	dirChan = make(chan string, 10000)
	fileChan = make(chan string, 10000)

	up = upyun.NewUpYun(&upyun.UpYunConfig{
		Bucket:   bucket,
		Operator: username,
		Password: password,
	})

	dirs := strings.Split(dir, ",")

	if filename != "" {
		content, _ := ioutil.ReadFile(filename)
		for _, l := range strings.Split(string(content), "\n") {
			if l != "" {
				dirs = append(dirs, l)
			}
		}
	}

	log.Println(dirs)

	for {
		dirs = doDeleteV2(dirs)
		if len(dirs) == 0 {
			return
		}
	}
}
